from falu.generic.get_api_request import GetApiRequest


class Wallet(GetApiRequest):
    """
    This is an object representing your Falu wallet. You can retrieve it to see the balance currently on your Falu account.
    You can also retrieve the wallet history, which contains a list of transactions that contributed to the balance (requests, payments, transfers, and so forth).
    """

    @classmethod
    def get_wallet(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve wallet

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/wallet",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.generic.get_api_request import GetApiRequest


class WalletTransaction(GetApiRequest):
    """
    Wallet transactions represent funds moving through your Falu account.
    They're created for every type of transaction that comes into or flows out of your Falu workspace balance..
    """

    @classmethod
    def get_transactions(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve wallet transactions

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/wallet_transactions",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_transaction(cls, transaction, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve wallet transaction

        :param transaction:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/wallet_transactions/{transaction}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class Customer(PostApiRequest, GetApiRequest, DeleteApiRequest, PatchApiRequest):
    """
    This object represents a customer of your business.
    It lets you track payments, transfers, and messages that belong to the same customer.
    """

    @classmethod
    def get_customers(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List customers

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/customers",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_customer(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create customer

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/customers",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_customer(cls, customer, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get customer

        :param customer:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/customers/{customer}".format(customer=customer),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_customer(cls, customer, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                        workspace=None, live: bool = None):
        """
        Update customer

        :param document:
        :param customer:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/customers/{customer}".format(customer=customer),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_customer(cls, customer, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Delete a customer. It cannot be undone.

        :param customer:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.delete(
            path="/customers/{customer}".format(customer=customer),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class MessageBatch(PostApiRequest, GetApiRequest, PatchApiRequest):

    @classmethod
    def send_bulk_messages(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                           live: bool = None):
        """
        Create a new message batch.
        You can send up to 1,000 messages in one API request.

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_batches",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message_batches(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List message batches

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_batches",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message_batch(cls, batch_id, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve a message batch

        :param batch_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_batches/{batch_id}".format(batch_id=batch_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_message_batch(cls, batch_id, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                             workspace=None, live: bool = None):
        """
        Retrieve a message batch

        :param document:
        :param batch_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/message_batches/{batch_id}".format(batch_id=batch_id),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_status(cls, batch_id, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve the status of a message batch

        :param batch_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_batches/{batch_id}/status".format(batch_id=batch_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def cancel_message_batch(cls, batch_id, api_key=None, idempotency_key: str = None, workspace=None,
                             live: bool = None):
        """
        A message batch can be cancelled when all its messages are in accepted status.

        :param batch_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_batches/{batch_id}/cancel".format(batch_id=batch_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def redact_message_batch(cls, batch_id, api_key=None, idempotency_key: str = None, workspace=None,
                             live: bool = None):
        """
        Redact a message batch to remove all collected information from Falu. This will redact the Message Batch,
        its Messages and all objects related to it, including Events, request logs, etc. A message batch can be redacted
        when its message are not in sending status. Redacting a message batch when all its messages are in accepted state
        will automatically cancel it and its messages.
        The redaction process may take up a week. When the redaction process is in progress, the MessageBatch's redaction.status
        field will be set to processing; when the process is finished, it will change to redacted and a message_batch.redacted
        event will be emitted. Redaction is irreversible. Redacted objects are still accessible in the API,
        but all the fields that contain personal data will be replaced by the string [redacted] or a similar placeholder.
        The metadata field will also be erased. Redacted objects cannot be updated or used for any purpose.

        :param batch_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_batches/{batch_id}/redact".format(batch_id=batch_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class PaymentRefund(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    PaymentRefund object allows you to refund a payment that has been previously been created but not yet refunded.
    Funds will be refunded to the payment instrument that was originally charged.
    Some payment providers have a time window after which refunds cannot happen.
    In this case you can initiate a Transfer to the customer with the appropriate amount.
    """

    @classmethod
    def get_payment_refunds(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List payment refunds

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/payment_refunds",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_payment_refund(cls, data, api_key=None, idempotency_key: str = None, workspace=None,
                              live: bool = None):
        """
        Create a payment refund

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/payment_refunds",
            api_key=api_key,
            data=cls.serialize(data),
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def retrieve_payment_refund(cls, refund, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        Retrieve a payment refund

        :param refund:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/payment_refunds/{refund}".format(refund=refund),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_payment_refund(cls, refund, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                              workspace=None, live: bool = None):
        """
        Update a payment refund

        :param document:
        :param refund:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/payment_refunds/{refund}".format(refund=refund),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



class FaluError(Exception):
    def __init__(self, message=None, status_code=None, error_code=None, problem=None):
        super(FaluError, self).__init__(message)

        self.status_code = status_code
        self.error_code = error_code
        self.problem = problem
        self._message = message

    def __str__(self):
        return self._message

    def __repr__(self):
        return "{class_name} => (Message:{message}, status: {status})".format(
            class_name=self.__class__.__name__,
            message=self._message,
            status=self.status_code
        )


class ApiError(FaluError):
    pass


class AuthenticationError(FaluError):
    pass


class ApiConnectionError(FaluError):
    pass



from falu.generic.post_api_request import PostApiRequest
from falu.generic.get_api_request import GetApiRequest


class Money(PostApiRequest, GetApiRequest):

    @classmethod
    def get_money_balance(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve balances for money services

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(path="/money_balances", api_key=api_key, idempotency_key=idempotency_key,
                       workspace=workspace, live=live)

    @classmethod
    def force_balance_refresh(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
          Balance request will be made upstream if the current balance is older than 5 min.

          :param api_key:
          :param idempotency_key:
          :param workspace:
          :param live:
          :return:
          """
        return cls.create(path="/money_balances/refresh", api_key=api_key, idempotency_key=idempotency_key,
                          workspace=workspace, live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest
from falu.list_options import MessageListOptions


class Messages(PostApiRequest, GetApiRequest, PatchApiRequest):

    @classmethod
    def get_messages(cls, options: MessageListOptions = None, api_key=None, idempotency_key: str = None, workspace=None,
                     live: bool = None):
        """
        List messages

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/messages",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def send_message(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create a message

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/messages",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message(cls, message_id, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve message

        :param message_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/messages/{message_id}".format(message_id=message_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_message(cls, message_id, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                       workspace=None, live: bool = None):
        """
        Update message

        :param document:
        :param message_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/messages/{message_id}".format(message_id=message_id),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def cancel_message(cls, message_id, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Cancel a message. A message can be cancelled when it is in accepted status.

        :param message_id:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/messages/{message_id}/cancel".format(message_id=message_id),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.generic.get_api_request import GetApiRequest
from falu.generic.post_api_request import PostApiRequest


class Event(PostApiRequest, GetApiRequest):
    """
    Events are our way of letting you know when something interesting happens in your workspace.
    When an interesting events occurs, we create a new Event object. For example, when a payment succeeds,
    we create a payment.succeeded event; and when a message delivery fails, we create an message.failed event.
    Note that many API requests may cause multiple events to be created. For example, if you create a new message,
    you will receive both a message.sent event and either a message.failed or message.delivered event.
    Events occur when the state of another API resource changes. The state of that resource at the time of the change is
    embedded in the event's data field. For example, a message.sent event will contain a message, and a payment.failed event will contain a payment.
    As with other API resources, you can use endpoints to retrieve an individual event or a list of events from the API.
    We also have a separate webhooks API for sending the Event objects directly to an endpoint on your server. Webhooks are managed in your workspace,
    and our Using Webhooks guide will help you get setup.
    NOTE: Access to events through the Retrieve Event API is guaranteed only for the last 30 days.
    """

    @classmethod
    def get_events(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get events

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/events",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_event(cls, event, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get event

        :param event:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/events/{event}".format(event=event),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class TerminalDevice(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    A TerminalDevice represents a physical device for performing identity verifications.
    """

    @classmethod
    def get_terminal_devices(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List terminal devices

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/terminals/devices",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_terminal_device(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create terminal device

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/terminals/devices",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_terminal_device(cls, device, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get terminal device

        :param device:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/terminals/devices/{device}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_terminal_device(cls, device, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                               workspace=None, live: bool = None):
        """
        Update terminal devices

        :param document:
        :param device:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/terminals/devices/{device}",
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_terminal_device(cls, device, api_key=None, idempotency_key: str = None, workspace=None,
                               live: bool = None):
        """
        Get terminal device

        :param device:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.delete(
            path=f"/terminals/devices/{device}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def handoff_terminal_device(cls, device, data, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        Hand-off processing of a visit to a terminal device

        :param data:
        :param device:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/terminals/devices/{device}/process_visit",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



class RangeFilteringOptions(object):
    """
     Standard options for range filtering
    """

    def __init__(self, less_than=None, less_than_or_equal_to=None, greater_than=None, greater_than_or_equal_to=None):
        self.less_than = less_than
        self.less_than_or_equal_to = less_than_or_equal_to
        self.greater_than = greater_than
        self.greater_than_or_equal_to = greater_than_or_equal_to



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class Payment(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    To initiate a payment from a customer, you create a Payment object.
    You can retrieve and refund individual payments as well as a list all payments.
    Payment objects can also be created if your workspace supports payment providers that allow customers to initiate the payments such as MPESA and EFT/RTGS.
    In this case, the provider may require you to authorize the payment before it is completed.
    """

    @classmethod
    def get_payments(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List payments

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/payments",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_payment(cls, payment, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve a payment

        :param payment:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/payments/{payment}".format(payment=payment),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_payment(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create a payment

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/payments",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_payment(cls, payment, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                       workspace=None, live: bool = None):
        """
        Update a payment

        :param document:
        :param payment:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/payments/{payment}".format(payment=payment),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.api_client import ApiClient


class GetApiRequest(ApiClient):

    @classmethod
    def get(cls, path, options=None, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None,
            params=None):
        return cls._execute(method="GET", path=path, options=options, key=api_key, idempotency_key=idempotency_key,
                            workspace=workspace, live=live, params=params)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class TransferReversal(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    TransferReversal object allows you to reverse a transfer that has been previously created but not yet reversed.
    Funds will be refunded your workspace.
    NOTE: Some providers have a time windows after which reversals cannot happen.
    """

    @classmethod
    def get_transfer_reversals(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List transfer reversal

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/transfer_reversal",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_transfer_reversal(cls, data, api_key=None, idempotency_key: str = None, workspace=None,
                                 live: bool = None):
        """
        Create transfer reversal

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/transfer_reversal",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def retrieve_transfer_reversal(cls, reversal, api_key=None, idempotency_key: str = None, workspace=None,
                                   live: bool = None):
        """
        Update transfer reversal

        :param reversal:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/transfer_reversal/{reversal}".format(reversal=reversal),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_transfer_reversal(cls, reversal, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                                 workspace=None, live: bool = None):
        """
        Update transfer reversal

        :param document:
        :param reversal:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/transfer_reversal/{reversal}".format(reversal=reversal),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.list_options import FileListOptions
from falu.generic.get_api_request import GetApiRequest
from falu.generic.post_api_request import PostApiRequest


class File(PostApiRequest, GetApiRequest):
    """
    This is an object representing a file hosted on Falu's servers.
    The file may have been uploaded by yourself using the create file request (for example, when uploading an identity document)
    or it may have been created by Falu (for example, a statement of accounts).
    """

    @classmethod
    def get_files(cls, options: FileListOptions = None, api_key=None, idempotency_key: str = None, workspace=None,
                  live: bool = None):
        """
        Get files

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/files",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def upload_files(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Upload files

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/files",
            data=data,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_file(cls, file, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get file

        :param file:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/files/{file}".format(file=file),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def download_file(cls, file, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Download file

        :param file:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        pass

    @classmethod
    def redact_file(cls, file, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Redact file content

        :param file:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/files/{file}/redact".format(file=file),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



import json

from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest
from falu.list_options import IdentityVerificationListOptions


class IdentityVerification(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    An IdentityVerification guides you through the process of collecting and verifying the identities of your users.
    It contains details such as what verification check to perform. Only create one IdentityVerification for each verification in your system.
    An IdentityVerification transitions through multiple statuses throughout its lifetime as it progresses through the verification flow.
    The IdentityVerification contains the user’s verified data after verification checks are complete.
    """

    @classmethod
    def get_identity_verifications(cls, options: IdentityVerificationListOptions = None, api_key=None,
                                   idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List identity verifications

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/identity/verifications",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_identity_verification(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                                     live: bool = None):
        """
        Create an identity verification

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/identity/verifications",
            data=json.dumps(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_identity_verification(cls, verification, api_key=None, idempotency_key: str = None, workspace=None,
                                  live: bool = None):
        """
        Get an identity verification

        :param verification:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/identity/verifications/{verification}".format(verification=verification),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_identity_verification(cls, verification, document: JsonPatchDocument, api_key=None,
                                     idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Update an identity verification

        :param document:
        :param verification:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/identity/verifications/{verification}".format(verification=verification),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def cancel_identity_verification(cls, verification, api_key=None, idempotency_key: str = None, workspace=None,
                                     live: bool = None):
        """
        Cancel an identity verification
        An identity verification can be cancelled when it is in input_required status.
        Once cancelled, future submission attempts are disabled. This cannot be undone!

        :param verification:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/identity/verifications/{verification}/cancel".format(verification=verification),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def redact_identity_verification(cls, verification, api_key=None, idempotency_key: str = None, workspace=None,
                                     live: bool = None):
        """
        Redact an identity verification
        Redact an identity verification to remove all collected information from Falu.
        This will redact the IdentityVerification and all objects related to it, including IdentityVerificationReports,
        Events, request logs, etc. An identity verification can be redacted when it is in input_required or verified status.
        Redacting an identity verification in input_required state will automatically cancel it.
        The redaction process may take up a week. When the redaction process is in progress, the IdentityVerification's redaction.status
        field will be set to processing; when the process is finished, it will change to redacted and an identity_verification.redacted event will be emitted.
        Redaction is irreversible. Redacted objects are still accessible in the API, but all the fields that contain personal data will be replaced by the string [redacted] or a similar placeholder.
        The metadata field will also be erased. Redacted objects cannot be updated or used for any purpose.

        :param verification:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/identity/verifications/{verification}/redact".format(verification=verification),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.api_client import ApiClient


class PatchApiRequest(ApiClient):

    @classmethod
    def patch(cls, path, data=None, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None,
              params=None):
        return cls._execute(method="PATCH", path=path, data=data, key=api_key, idempotency_key=idempotency_key,
                            workspace=workspace, live=live, params=params)



from falu.query_values import QueryValues
from falu.range_filtering_options import RangeFilteringOptions


class BasicListOptions(object):
    """
    Standard options for filtering and pagination in list operations.
    """

    def __init__(self, sorting=None, count=None, created: RangeFilteringOptions = None,
                 update: RangeFilteringOptions = None):
        self.sorting = sorting
        self.count = count
        self.created = created
        self.update = update

    def populate(self, values: QueryValues):
        if values is None: return

        values.add("sort", [self.sorting])
        values.add("count", self.count)
        values.add_query_values("created", QueryValues().fromRange(self.created))
        values.add_query_values("updated", QueryValues().fromRange(self.update))

class BasicListOptionsWithMoney(BasicListOptions):
    """
    Standard options for filtering and pagination in list operations with money.
    """

    def __init__(self, currency=None, amount: RangeFilteringOptions = None):
        super().__init__()
        self.currency = currency
        self.amount = amount

    def populate(self, values: QueryValues):
        super().populate(values)

        values.add("currency", self.currency).add_query_values("amount", QueryValues().fromRange(self.amount))


class MessageListOptions(BasicListOptions):
    """
    Options for filtering and pagination of messages.
    """

    def __init__(self, delivered: RangeFilteringOptions = None, status=None):
        super().__init__()
        self.delivered = delivered
        self.status = status

    def populate(self, values: QueryValues):
        super().populate(values)

        values \
            .add("delivered", QueryValues().fromRange(self.delivered)) \
            .add("status", self.status)


class IdentityVerificationListOptions(BasicListOptions):
    """
    Options for filtering identity verifications
    """

    def __init__(self, status=None, document_type=None, customer=None):
        super().__init__()
        self.status = status
        self.document_type = document_type
        self.customer = customer

    def populate(self, values: QueryValues):
        super().populate(values)
        values.add("status", self.status)
        values.add("type", self.document_type)
        values.add("customer", self.customer)

class IdentityVerificationReportsListOptions(BasicListOptions):
    """
    Options for filtering identity verification reports
    """

    def __init__(self, verification=None):
        super().__init__()
        self.verification = verification

    def populate(self, values: QueryValues):
        super().populate(values)
        values.add("verification", self.verification)


class FileListOptions(BasicListOptions):
    """
    Options for filtering and pagination of list files
    """

    def __init__(self, purpose=None):
        super().__init__()
        self.purpose = purpose

    def populate(self, values: QueryValues):
        super().populate(values)
        values.add("purpose", self.purpose)


class FileLinksListOptions(BasicListOptions):
    """
    Options for filtering and pagination of list file links.
    """

    def __init__(self, file=None):
        super().__init__()
        self.file = file

    def populate(self, values: QueryValues):
        super().populate(values)
        values.add("file", self.file)


class TransferListOptions(BasicListOptionsWithMoney):
    """
    Options for filtering and pagination of transfers.
    """

    def __init__(self, status=None):
        super().__init__()
        self.status = status

    def populate(self, values: QueryValues):
        super().populate(values)
        values.add("status", self.status)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class Visit(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    A Visit represents a record that a Visitor did visit a given VisitorDestination.
    """

    @classmethod
    def get_visits(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List visits

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/visits/visits",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_visit(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create visit

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/visits/visits",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_visit(cls, visit, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve a visit

        :param visit:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/visits/visits/{visit}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_visit(cls, visit, document: JsonPatchDocument, api_key=None, idempotency_key: str = None, workspace=None,
                     live: bool = None):
        """
        Update a visit

        :param document:
        :param visit:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/visits/visits/{visit}",
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def start_visit(cls, visit, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Start visit

        :param visit:
        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/visits/visits/{visit}/start",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def end_visit(cls, visit, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        End visit

        :param visit:
        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/visits/visits/{visit}/end",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def redact_visit(cls, visit, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Redact visit

        :param visit:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/visits/visits/{visit}/redact",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from datetime import datetime

from falu.range_filtering_options import RangeFilteringOptions


class QueryValues(object):
    """
    Helper for handling query parameters
    """

    def __init__(self, values=None):
        if values is None:
            values = {}
        self.values = values

    def add(self, key, value=None):
        if value is None:
            return self

        elif isinstance(value, datetime):
            self._add(key, [value])
        else:
            self._add(key, ["{}".format(value)])
        return self

    def _add(self, key, values):
        if values is not None and len(values) > 0:
            self.values[key] = tuple(values)
        return self

    def add_query_values(self, prop, other):
        if other is None:
            return self

        if prop is None or len(prop) < 1:
            return self

        if isinstance(other, QueryValues):
            for key, value in other.values.items():
                if value is not None:
                    self.add(prop + "." + key, value)
        else:
            return self

    def fromRange(self, options: RangeFilteringOptions):
        if options is not None:
            self.add("lt", options.less_than)
            self.add("lte", options.less_than_or_equal_to)
            self.add("gt", options.greater_than)
            self.add("gte", options.greater_than_or_equal_to)
        return self

    def get_keys(self):
        return self.values.keys()


import json

from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.post_api_request import PostApiRequest


class MessageSuppressions(PostApiRequest, GetApiRequest, DeleteApiRequest):
    """
    A MessageSuppression object allows you to manage the phone number destinations that are not currently active for sending to.
    """

    @classmethod
    def get_message_suppressions(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List message suppression

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_suppressions",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_message_suppression(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                                   live: bool = None):
        """
        Create message suppression

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_suppressions",
            data=json.dumps(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message_suppression(cls, suppression, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        List message suppression

        :param suppression:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path=f"/message_suppressions/{suppression}".format(suppression=suppression),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_message_suppression(cls, suppression, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        List message suppression

        :param suppression:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.delete(
            path=f"/message_suppressions/{suppression}".format(suppression=suppression),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class Visitor(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    A Visitor represents an individual who visited the premises.
    """

    @classmethod
    def get_visitors(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List visitor

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/visits/visitors",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_visitor(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create visitor

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/visits/visitors",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_visitor(cls, visitor, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get terminal device

        :param visitor:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/visits/visitors/{visitor}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_visitor(cls, visitor, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                       workspace=None, live: bool = None):
        """
        Update visitor

        :param document:
        :param visitor:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/visits/visitors/{visitor}",
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_visitor(cls, visitor, api_key=None, idempotency_key: str = None, workspace=None,
                       live: bool = None):
        """
        Delete visitor

        :param visitor:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.delete(
            path=f"/visits/visitors/{visitor}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def redact_visitor(cls, visitor, api_key=None, idempotency_key: str = None, workspace=None,
                       live: bool = None):
        """
        Redact visitor

        :param visitor:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/visits/visitors/{visitor}/redact",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest
from falu.list_options import BasicListOptions


class WebhookEndpoint(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    You can configure webhook endpoints via the API to be notified about events that happen in your Falu workspace.
    Most users configure webhooks from the dashboard, which provides a user interface to registering and testing your webhook endpoints.
    """

    @classmethod
    def get_webhook_endpoints(cls, options: BasicListOptions = None, api_key=None, idempotency_key: str = None,
                              workspace=None,
                              live: bool = None):
        """
        List webhook endpoints

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/webhooks/endpoint",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_webhook_endpoint(cls, data, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        Create webhook endpoint

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/webhooks/endpoint",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_webhook_endpoint(cls, webhook_endpoint, api_key=None, idempotency_key: str = None, workspace=None,
                             live: bool = None):
        """
        Get webhook endpoint

        :param webhook_endpoint:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/webhooks/endpoint/{webhook_endpoint}".format(webhook_endpoint=webhook_endpoint),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_webhook(cls, webhook_endpoint, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                       workspace=None, live: bool = None):
        """
        Update webhook endpoint

        :param document:
        :param webhook_endpoint:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/webhooks/endpoint/{webhook_endpoint}".format(webhook_endpoint=webhook_endpoint),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_webhook_endpoint(cls, webhook_endpoint, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        Get webhook endpoint

        :param webhook_endpoint:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.delete(
            path=f"/webhooks/endpoint/{webhook_endpoint}".format(webhook_endpoint=webhook_endpoint),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def roll_webhook_endpoint(cls, webhook_endpoint, data, api_key=None, idempotency_key: str = None, workspace=None,
                     live: bool = None):
        """
        Roll a webhook endpoint secret

        :param data:
        :param webhook_endpoint:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/webhooks/endpoint/{webhook_endpoint}".format(webhook_endpoint=webhook_endpoint),
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



import json

from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class MessageStream(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):

    @classmethod
    def get_message_streams(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List message streams

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_streams",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_message_stream(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                              live: bool = None):
        """
        Create message stream

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_streams",
            data=json.dumps(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message_stream(cls, stream, api_key=None, idempotency_key: str = None, workspace=None,
                           live: bool = None):
        """
        Get message stream

        :param stream:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/message_streams/{stream}".format(stream=stream),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_message_stream(cls, stream, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                              workspace=None, live: bool = None):
        """
        Update message stream

        :param document:
        :param stream:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/message_streams/{stream}".format(stream=stream),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_message_stream(cls, stream, api_key=None, idempotency_key: str = None, workspace=None,
                              live: bool = None):
        """
        Delete message stream

        :param stream:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        cls.delete(
            path="/message_streams/{stream}".format(stream=stream),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def archive_message_stream(cls, stream, api_key=None, idempotency_key: str = None, workspace=None,
                               live: bool = None):
        """
        Archive message stream

        :param stream:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_streams/{stream}/archive".format(stream=stream),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def unarchive_message_stream(cls, stream, api_key=None, idempotency_key: str = None, workspace=None,
                                 live: bool = None):
        """
        Unarchive message stream

        :param stream:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.create(
            path="/message_streams/{stream}/unarchive".format(stream=stream),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



class JsonPatchDocument(object):

    def __init__(self):
        self.operations = []

    def add(self, path: str, value):
        self.operations.append(AddOperation(path=path, value=value).__dict__)
        return self

    def remove(self, path):
        self.operations.append(RemoveOperation(path=path).__dict__)
        return self

    def test(self, path: str, value):
        self.operations.append(TestOperation(path=path, value=value).__dict__)
        return self

    def replace(self, path: str, value):
        self.operations.append(ReplaceOperation(path=path, value=value).__dict__)
        return self

    def get_operations(self):
        return self.operations


class PathOperation(object):

    def __init__(self, op):
        self.op = op


class AddOperation(PathOperation):
    def __init__(self, path: str, value):
        super().__init__("add")
        self.path = path
        self.value = value

class RemoveOperation(PathOperation):
    def __init__(self, path: str):
        super().__init__("remove")
        self.path = path


class ReplaceOperation(PathOperation):
    def __init__(self, path: str, value):
        super().__init__("replace")
        self.path = path
        self.value = value


class TestOperation(PathOperation):
    def __init__(self, path: str, value):
        super().__init__("test")
        self.path = path
        self.value = value



import json
import platform

import requests
from requests import Response

from falu import errors
from falu import version
from falu.client.falu_model import FaluModel
from falu.client.falu_model import deserialize_falu_response
from falu.list_options import BasicListOptions
from falu.query_values import QueryValues


class ApiClient(FaluModel):
    def __init__(self):
        super().__init__()
        self.base_url = 'https://api.falu.io/v1'

    @classmethod
    def _execute(cls, method, path, data=None, options: BasicListOptions = None, key=None, idempotency_key: str = None,
                 workspace=None, live: bool = None, params=None):
        params = cls._generate_params(options, params)
        client = ApiClient()
        return client.execute(method, path, data, key, idempotency_key, workspace, live, params)

    def execute(self, method, path, data=None, key=None, idempotency_key: str = None, workspace=None, live: bool = None,
                params=None):

        url = self._build_url(path)
        headers = self.build_headers(key=key, method=method, idempotency_key=idempotency_key, workspace=workspace,
                                     live=live)

        response = requests.request(method=method, url=url, headers=headers, data=data, params=params)
        return self.response_handler(response)

    def _build_url(self, path):
        return "%s%s" % (self.base_url, path)

    @staticmethod
    def _generate_params(options: BasicListOptions = None, params=None):
        args = QueryValues()
        if options is not None:
            options.populate(args)
            return args.values.update(params)
        return params

    @staticmethod
    def build_headers(key=None, method=None, idempotency_key: str = None, workspace: str = None, live: bool = None):
        if key:
            api_key = key
        else:
            from falu import api_key
            api_key = api_key

        if api_key is None:
            raise errors.AuthenticationError(
                "API Key is required!\n"
                "(Hint: define your api key 'falu.api_key = <KEY>')\n"
                "For more details see https://falu.io"
            )

        user_agent = "falu-python/{version}; (Python {lang_version};{platform}; system ({system};{node};{release};{machine}))".format(
            version=version.VERSION, lang_version=platform.python_version(), platform=platform.platform(),
            system=platform.system(),
            node=platform.node(),
            release=platform.release(),
            machine=platform.machine()
        )

        headers = {
            "Authorization": "Bearer %s" % api_key,
            "X-Falu-Version": "2022-09-01",
            'Accept': 'application/json',
            'User-Agent': user_agent
        }

        if method == "POST":
            headers['Content-type'] = 'application/json'

        if idempotency_key is not None:
            headers['X-Idempotency-Key'] = idempotency_key

        if workspace is not None:
            headers['X-Workspace-Id'] = workspace

        live = live is not None and live
        headers['X-Live-Mode'] = str(live)
        return headers

    def response_handler(self, response: Response):
        code = response.status_code
        resource = None
        error = None

        if response is not None:
            if code == 401:
                error = errors.AuthenticationError(message=response.reason, status_code=code)

            if code == 400 and response.content:
                problem = response.json()
                error = errors.ApiError(message=problem["detail"], status_code=code, problem=problem)

            if code in [200, 201, 204] and response.content:
                resource = response.json()
        else:
            raise errors.ApiConnectionError("IOException during API request to {url}. " +
                                            "Please check your internet connection and try again. " +
                                            "If this problem persists, let us know at support@falu.io."
                                            .format(url=response.url))

        return self._response_handler(code, response.headers, resource, error)

    @staticmethod
    def _response_handler(code, headers, resource, error):
        return deserialize_falu_response(code, headers, resource), error

    @classmethod
    def serialize(cls, data: dict):
        return json.dumps(data)



from falu.generic.get_api_request import GetApiRequest
from falu.list_options import IdentityVerificationReportsListOptions


class IdentityVerificationReport(GetApiRequest):
    """
    An IdentityVerificationReport is the result of an attempt to collect and verify data from a user.
    You can find the result of each verification check performed in the appropriate node: document, id_number, tax_number.
    Each IdentityVerificationReport contains a copy of any data collected by the user as well as reference identifiers which can be used to access collected images.
    IdentityVerificationReports are created automatically via the IdentityVerification API.
    """

    @classmethod
    def get_identity_verification_reports(cls, options: IdentityVerificationReportsListOptions = None, api_key=None,
                                          idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List identity verification reports

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/identity/verification_reports",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_identity_verification_report(cls, report, api_key=None, idempotency_key: str = None, workspace=None,
                                         live: bool = None):
        """
        List identity verification reports

        :param report:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/identity/verification_reports/{report}".format(report=report),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.services.customers import Customer
from falu.services.events import Event
from falu.services.file_links import FileLink
from falu.services.files import File
from falu.services.identity_verification_reports import IdentityVerificationReport
from falu.services.identity_verifications import IdentityVerification
from falu.services.message_batches import MessageBatch
from falu.services.message_streams import MessageStream
from falu.services.message_suppressions import MessageSuppressions
from falu.services.message_templates import MessageTemplates
from falu.services.messages import Messages
from falu.services.money import Money
from falu.services.payment_authorizations import PaymentAuthorization
from falu.services.payment_refunds import PaymentRefund
from falu.services.payments import Payment
from falu.services.terminal_configurations import TerminalConfiguration
from falu.services.terminal_devices import TerminalDevice
from falu.services.terminal_locations import TerminalLocation
from falu.services.transfer_reversals import TransferReversal
from falu.services.transfers import Transfer
from falu.services.visitor_destinations import VisitorDestination
from falu.services.visitors import Visitor
from falu.services.visits import Visit
from falu.services.wallet import Wallet
from falu.services.wallet_transactions import WalletTransaction
from falu.services.webhook_endpoints import WebhookEndpoint



from __future__ import absolute_import, division, print_function

# Bindings for Falu
# API docs at https://falu.io/docs/api

api_key = None
app_information = None

# Set to either 'debug' or 'info',
log = None

# API services
from falu.services import *



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class MessageTemplates(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
     Deleting a suppression allows you to resume sending messages to the given destination.
     Suppression with spam_complaint reason cannot be deleted.
    """

    @classmethod
    def get_message_templates(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
           List message templates

           :param api_key:
           :param idempotency_key:
           :param workspace:
           :param live:
           :return:
        """
        return cls.get(
            path="/message_templates",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_message_template(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
           Create message template

           :param data:
           :param api_key:
           :param idempotency_key:
           :param workspace:
           :param live:
           :return:
        """
        return cls.create(
            path="/message_templates",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_message_template(cls, template, api_key=None, idempotency_key: str = None, workspace=None,
                             live: bool = None):
        """
          Get message template

          :param template:
          :param api_key:
          :param idempotency_key:
          :param workspace:
          :param live:
          :return:
        """
        return cls.get(
            path=f"/message_templates/{template}".format(template=template),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_message_template(cls, template, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                                workspace=None, live: bool = None):
        """
          Update message template

          :param document:
          :param template:
          :param api_key:
          :param idempotency_key:
          :param workspace:
          :param live:
          :return:
        """
        return cls.patch(
            path=f"/message_templates/{template}".format(template=template),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_message_template(cls, template, api_key=None, idempotency_key: str = None, workspace=None,
                                live: bool = None):
        """
        Delete message template

        :param template:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.delete(
            path=f"/message_templates/{template}".format(template=template),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def validate_message_template(cls, data: dict, api_key=None, idempotency_key: str = None, workspace=None,
                                  live: bool = None):
        """
          Validate message template

          :param data:
          :param api_key:
          :param idempotency_key:
          :param workspace:
          :param live:
          :return:
        """
        return cls.create(
            path="/message_templates/validate",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class PaymentAuthorization(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    When a payment provider supports customer initiated payments and a customer initiates a payment, a Payment Authorization object is created.
    Authorizations must be approved for the payment to be completed successfully.
    """

    @classmethod
    def get_payment_authorizations(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List payment authorizations

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/payments_authorizations",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_payment_authorization(cls, payment_authorization, api_key=None, idempotency_key: str = None, workspace=None,
                                  live: bool = None):
        """
        Retrieve a payment authorization

        :param payment_authorization:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/payments_authorizations/{payment_authorization}".format(
                payment_authorization=payment_authorization),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_payment_authorization(cls, payment_authorization, document: JsonPatchDocument, api_key=None,
                                     idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Update a payment authorization

        :param document:
        :param payment_authorization:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/payments_authorizations/{payment_authorization}".format(
                payment_authorization=payment_authorization),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def approve_payment_authorization(cls, payment_authorization, data=None, api_key=None, idempotency_key: str = None,
                                      workspace=None, live: bool = None):
        """
        Approve a payment authorization

        :param data:
        :param payment_authorization:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/payments_authorizations/{payment_authorization}/approve".format(
                payment_authorization=payment_authorization),
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def decline_payment_authorization(cls, payment_authorization, data=None, api_key=None, idempotency_key: str = None,
                                      workspace=None, live: bool = None):
        """
        Decline a payment authorization

        :param data:
        :param payment_authorization:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path=f"/payments_authorizations/{payment_authorization}/decline".format(
                payment_authorization=payment_authorization),
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)




from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class TerminalConfiguration(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    A TerminalConfiguration object represents how features should be configured for terminal devices.
    """

    @classmethod
    def get_configurations(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List terminal configurations

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/terminals/configurations",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_configuration(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create a terminal configuration

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/terminals/configurations",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_configuration(cls, config, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get terminal configuration

        :param config:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/terminals/configurations/{config}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_configuration(cls, config, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                             workspace=None, live: bool = None):
        """
        Update terminal configuration

        :param document:
        :param config:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/terminals/configurations/{config}",
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_configuration(cls, config, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get terminal configuration

        :param config:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.delete(
            path=f"/terminals/configurations/{config}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



from falu.client.api_client import ApiClient

class DeleteApiRequest(ApiClient):

    @classmethod
    def delete(cls, path, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None,
               params=None):
        return cls._execute(method="DELETE", path=path, key=api_key, idempotency_key=idempotency_key,
                            workspace=workspace, live=live, params=params)



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest
from falu.list_options import TransferListOptions


class Transfer(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    A Transfer object is created when you move money from your business either to another business or to a customer.
    """

    @classmethod
    def get_transfers(cls, options: TransferListOptions = None, api_key=None, idempotency_key: str = None,
                      workspace=None, live: bool = None):
        """
        List transfers

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/transfers",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_transfer(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create a transfer

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/transfers",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def retrieve_transfer(cls, transfer, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Retrieve a transfer

        :param transfer:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.get(
            path="/transfers/{transfer}".format(transfer=transfer),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_transfer(cls, transfer, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                        workspace=None, live: bool = None):
        """
        Update a transfer

        :param document:
        :param transfer:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path="/transfers/{transfer}".format(transfer=transfer),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



import os

from setuptools import setup, find_packages

locale = os.path.abspath(os.path.dirname(__file__))

version = {}
with open(os.path.join(locale, "falu", "version.py"), encoding="utf-8") as f:
    exec(f.read(), version)

setup(
    name='falu',
    version=version["VERSION"],
    description='The official Falu Python library',
    long_description=open("long_description.rst").read(),
    long_description_content_type="text/x-rst",
    url='https://github.com/tinglesoftware/falu-python',
    author='Falu',
    author_email='info@tingle.software',
    keywords=["falu api payments"],
    zip_safe=False,
    license='MIT',
    python_requires=">=3.7",
    packages=find_packages(exclude=["tests", "tests.*"]),
    install_requires=[
        "pytest>=7.0.0",
        'requests >= 2.20; python_version >= "3.0"',
        'responses >=0.23',
        "urllib3 == 1.26.14"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    setup_requires=["wheel"],
)



import json

from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest
from falu.list_options import BasicListOptions


class FileLink(PostApiRequest, GetApiRequest, PatchApiRequest):
    """
    To share the contents of an File object with non-Falu users, you can create an FileLink.
    It contains a URL that can be used to retrieve the contents of the file without authentication.
    """

    @classmethod
    def get_file_links(cls, options: BasicListOptions = None, api_key=None, idempotency_key: str = None, workspace=None,
                       live: bool = None):
        """
        Get file links

        :param options:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/file_links",
            options=options,
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_file_link(cls, data, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Create file links

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/file_links",
            data=json.dumps(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_file_link(cls, link, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Get file links

        :param link:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/file_links/{link}".format(link=link),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_file_link(cls, link, document: JsonPatchDocument, api_key=None, idempotency_key: str = None,
                         workspace=None, live: bool = None):
        """
        Update file links

        :param document:
        :param link:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/file_links/{link}".format(link=link),
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)



def deserialize_falu_response(code, headers, response):
    """
    If we get a server a json response from Falu's server, the response needs to be converted to a Falu Class object.
    :param headers:
    :param code:
    :param response: json response from the server.
    :return: FaluModel
    """

    if isinstance(response, list):
        return [deserialize_falu_response(code, headers, r) for r in response]
    elif isinstance(response, dict):
        model = FaluModel
        resource = model.create_object(code, headers, response)
        return resource


class FaluModel(object):
    def __init__(self, code=None, headers=None, resource=None):
        super(FaluModel, self).__init__()
        self.code = code
        self.headers = headers
        self.resource = resource

    @classmethod
    def create_object(cls, code=None, headers=None, resource=None):
        instance = cls(
            code,
            headers,
            resource
        )
        instance._create_from(resource)
        return instance

    def _create_from(self, resource):
        for k, v in resource.items():
            super(FaluModel, self).__setattr__(k, v)
        return resource



from falu.client.json_patch_document import JsonPatchDocument
from falu.generic.delete_api_request import DeleteApiRequest
from falu.generic.get_api_request import GetApiRequest
from falu.generic.patch_api_request import PatchApiRequest
from falu.generic.post_api_request import PostApiRequest


class TerminalLocation(PostApiRequest, GetApiRequest, PatchApiRequest, DeleteApiRequest):
    """
    A TerminalLocation represents a grouping of devices.
    """

    @classmethod
    def get_terminal_locations(cls, api_key=None, idempotency_key: str = None, workspace=None, live: bool = None):
        """
        List terminal locations

        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path="/terminals/locations",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def create_terminal_location(cls, data, api_key=None, idempotency_key: str = None, workspace=None,
                                 live: bool = None):
        """
        List terminal locations

        :param data:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.create(
            path="/terminals/locations",
            data=cls.serialize(data),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def get_terminal_location(cls, terminal_location, api_key=None, idempotency_key: str = None, workspace=None,
                              live: bool = None):
        """
        Get terminal location

        :param terminal_location:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """

        return cls.get(
            path=f"/terminals/locations/{terminal_location}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def update_terminal_location(cls, terminal_location, document: JsonPatchDocument, api_key=None,
                                 idempotency_key: str = None, workspace=None, live: bool = None):
        """
        Update terminal location

        :param document:
        :param terminal_location:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.patch(
            path=f"/terminals/locations/{terminal_location}",
            data=cls.serialize(document.operations),
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)

    @classmethod
    def delete_terminal_location(cls, terminal_location, api_key=None, idempotency_key: str = None, workspace=None,
                                 live: bool = None):
        """
        Delete terminal location

        :param terminal_location:
        :param api_key:
        :param idempotency_key:
        :param workspace:
        :param live:
        :return:
        """
        return cls.delete(
            path=f"/terminals/locations/{terminal_location}",
            api_key=api_key,
            idempotency_key=idempotency_key,
            workspace=workspace,
            live=live)
